library(reshape2)
library(e1071)

args <- commandArgs(trailingOnly = TRUE)

projectName = args[1]
trainSetName = args[2] # output name
useSS = (args[3] == "ss")
rDataFolder = args[4]

if (useSS)
{
	load(paste(rDataFolder,"/fit_",trainSetName,".RData",sep = ""))
} else {
	load(paste(rDataFolder,"/fit_without_SS_",trainSetName,".RData",sep = ""))
}

hmmScanFile = paste(projectName,".hmmOut",sep = "")
CompFile = paste(projectName,".composition",sep = "")
if (useSS)
{
	SSFile = paste(projectName,".ss",sep = "")
	ACCFile = paste(projectName,".acc",sep = "")
}

# Load the HMM scan file
zTest = read.table(hmmScanFile)
zTestL = zTest[zTest$V2=="1st",]
zTestR = zTest[zTest$V2=="2nd",]
zTest = merge(zTestL,zTestR,by=c("V1","V4")) # Keep only hits that have the left and right hits (of the two segements of the backbone)
zTestL <- NULL
zTestR <- NULL
zTest$EVal <- zTest$V6.x*zTest$V6.y # Supposing that the two hits are independent, se the E-value as the product of the E-val for the Left and the Right segment respectively
colnames(zTest)[2] = "ID"
colnames(zTest)[1] = "HMMNumber"

# Load composition of the sequences in the FASTA file
zTestComp = read.table(CompFile,sep=";")
rownames(zTestComp) = zTestComp$V1
zTestComp = zTestComp[2:ncol(zTestComp)] / rowSums(zTestComp[2:ncol(zTestComp)]) # Average composition

if (useSS)
{
	#Load SS prediction
	zTestSS = read.table(SSFile,sep=";")
	rownames(zTestSS) = zTestSS$V1
	zTestSS = zTestSS[2:ncol(zTestSS)] / rowSums(zTestSS[2:ncol(zTestSS)])
	colnames(zTestSS) = c('H','E','C')
    
	#Load ACC pred
	zTestACC = read.table(ACCFile,sep=";")
	rownames(zTestACC) = zTestACC$V1
	zTestACC = zTestACC[2:ncol(zTestACC)] / rowSums(zTestACC[2:ncol(zTestACC)])
}

# Make a clean list of HMM hits
toSv = rbind(zTest[,c("ID","HMMNumber","EVal")])
toSv = toSv[complete.cases(toSv),]
toSv$HMMNumber = factor(toSv$HMMNumber,levels = hmmNames)
toSv$ID = factor(toSv$ID,levels = unique(rownames(zTestComp)))

# transform hit list into a matrix : columns are contact fragments, rows are sequences in the FASTA file, keeping the best hit par HMM per sequence
svTest = as.data.frame(acast(toSv,ID~HMMNumber,value.var = "EVal",fun.aggregate=min,drop=FALSE))
svTest[svTest == Inf] <- 1000 # 10 times the maximum EValue which is 100 in HMMer
svTest = svTest[,hmmNames] # Put the order of columns as same as for in the training

# Adding boosted hmm rank feature
BoostedRankFeatNumber = 1
finalRankTest = rowSums( t(as.vector(alpha) * t(svTest[,1:length(alpha)])) ) # Alpha contains the trained coefficients resulting of boosting
featMatTest = as.matrix(finalRankTest)
colnames(featMatTest)[BoostedRankFeatNumber] = "BoostedRank"

# Add bestHit in the features:
BestHitFeatNumber = ncol(featMatTest) + 1
scoresTest = data.frame(V12 = apply(X=svTest[,1:80],FUN = min,MARGIN = 1))
featMatTest = cbind(featMatTest, scoresTest )
colnames(featMatTest)[BestHitFeatNumber] = "BestHit"

# Add Nb Hit/seq in the features:
HitPerSeqFeatNumber = ncol(featMatTest) + 1
featMatTest = cbind(featMatTest,rowSums(svTest<max(svTest*0.9)) )
colnames(featMatTest)[HitPerSeqFeatNumber] = "HitPerSeq"

if (useSS)
{
	# Add SS in the features:
	SSFeatNumber = (ncol(featMatTest) + 1):(ncol(featMatTest) + 2)
	featMatTest = cbind(featMatTest,zTestSS[match(rownames(featMatTest),rownames(zTestSS)),c(1,2)])
    
	# Add ACC in the features:
	#ACCFeatNumber = ncol(featMatTest) + 1
	#featMatTest = cbind(featMatTest,zTestACC[match(rownames(featMatTest),rownames(zTestACC)),"V2"])
	#colnames(featMatTest)[ACCFeatNumber] = "ACC"
}

# Add composition in the features:
CompFeatNumber = (ncol(featMatTest) + 1):(ncol(featMatTest) + 20)
featMatTest = cbind(featMatTest,zTestComp[match(rownames(featMatTest),rownames(zTestComp)),])

# Add all 100 best HMM hits
#HMMHitsFeatNumber = max(CompFeatNumber) + 1
#featMatTest = cbind(featMatTest,svTest[match(rownames(featMatTest),rownames(svTest)),1:100])

# List of features used by VIRALpro, here we keep all features we defined previously
selectedFeat = colnames(featMatTest)

# Select only kept features
currentSvTest = featMatTest[,selectedFeat]

# Prediction by the SVM
p = predict(fit,currentSvTest,decision.values=TRUE)
scores = attr(p, "decision.values")

scores[,1] = -scores[,1] # To have positive instances with positive scores.
if(colnames(scores)[1] != "0/1") # Prevent for potential inversion due to e1071 library
{
	scores[,1] = -scores[,1]
}
scores[,1] = -scores[,1]

scores = as.data.frame(scores)
colnames(scores) = "score"
scores$ID = rownames(scores)

# Finally, write out the results
write.table(scores[order(scores[,"score"]),c("ID","score")],file = paste(projectName,"_",trainSetName,".scores.csv",sep=""),row.names=F)

